/* la48d.c */
/* La48d programming example */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <termio.h>
#include <stropts.h>

#include <sys/select.h>
#include <sys/ioctl.h>

#include "lapci.h"
#include "plx9050.h"

// IOMEM =1 - use IO space, IOMEM = 0 - use memory space
#define IOMEM 1

/* Open LaPCI device */
LAPCI_HANDLE LaPCIOpen(char *Name, int DevNum)
{
	int i,j,d;
	char devName[12];
	LaPCI_dev_info devInfo;

	j=0;
	for(i=0;i<8;i++)
	{
		sprintf(devName,"/dev/lapci%d",i);
		d = open(devName, O_RDWR);
		if(d==-1)
			continue;
		ioctl(d, LAPCI_IOCTL_GETINFO, &devInfo);
		printf("%08x PLX%x %s\n",devInfo.subsys_id, devInfo.PLXrev, devInfo.name);
		if(strcmp(devInfo.name,Name)!=0)
		{
			close(d);
			continue;
		}
		j++;
		if(j==DevNum)
			return (LAPCI_HANDLE) d;
		close(d);
	}
	return -1;
}

/* Write port A */
int WritePortA(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=0;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Write port B */
int WritePortB(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=1;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Write port C */
int WritePortC(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=2;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Write control word */
int WriteCW(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=3;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Read port A */
unsigned char ReadPortA(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=0;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[3];
}

/* Read port B */
unsigned char ReadPortB(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=1;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Read port C */
unsigned char ReadPortC(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=2;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Read control word */
unsigned char ReadCW(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=2;
	IOStruct.Address=3;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=1;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Write port A1 */
int WritePortA1(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=0;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Write port B1 */
int WritePortB1(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=1;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Write port C1 */
int WritePortC1(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=2;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Write control word1 */
int WriteCW1(LAPCI_HANDLE DevHandle, unsigned char Data)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=3;
	IOStruct.Data.lVal=(unsigned long)(Data);

	if(IOMEM)
		return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		return ioctl(DevHandle, LAPCI_IOCTL_OUTMEM, &IOStruct);
	}
}

/* Read port A1 */
unsigned char ReadPortA1(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=0;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Read port B1 */
unsigned char ReadPortB1(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=1;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Read port C1 */
unsigned char ReadPortC1(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=2;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Read control word1 */
unsigned char ReadCW1(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=1;
	IOStruct.Region=1;
	IOStruct.Address=3;
	
	if(IOMEM)
		ioctl(DevHandle, LAPCI_IOCTL_INP, &IOStruct);
	else
	{
		IOStruct.Region=0;
		ioctl(DevHandle, LAPCI_IOCTL_INMEM, &IOStruct);
	}

	return IOStruct.Data.cVal[0];
}

/* Init PLX chip */
int initPLX(LAPCI_HANDLE DevHandle)
{
	LaPCI_IO_struct IOStruct;

	IOStruct.Size=4;
	IOStruct.Region=0;
	IOStruct.Address=PLX9050_INTCSR;
	IOStruct.Data.lVal=PLX9050_ICS_LI1P|PLX9050_ICS_LI2P;

	return ioctl(DevHandle, LAPCI_IOCTL_OUTP, &IOStruct);
}

//----------------------------------------------------------
int main(int argc, char **argv)
{
	LAPCI_HANDLE d;
	int stat, count;
	
	d = LaPCIOpen("LA48D",1);
	if(d < 0)
	{
		printf("Can't open La48d device\n");
		return d;
	}
	printf("Device La48d found\n");
	initPLX(d);

	WriteCW(d,0x80);		/* Write control word mode 0 , A,B,C - output */
	WritePortC(d,0x0);
	WriteCW1(d,0x9f);		/* Write control word1 mode 0 for A1, mode1 for B1, A1,B1,C1 - input */
	WriteCW1(d,0x05);		/* Enable  INTEb interrupt */
	WriteCW1(d,0x08);		/* Disable INTEa interrupt */

	stat=2;
	ioctl(d, LAPCI_IOCTL_ENAIRQ, &stat); /* Enable PLX IRQ2 */

	for(count=0; count<8; count++)
	{
		WritePortB(d,1<<count);
		WritePortA(d,0xaa);	
		WriteCW(d,0x5);
		
		usleep(1000);
		ioctl(d, LAPCI_IOCTL_GETIRQ, &stat);
		if(stat & (PLX9050_ICS_LI1S|PLX9050_ICS_LI2S)) 
		{
			printf("IRQStat=%x\n",stat);
			printf("Port A1 = %x\n",ReadPortA1(d));
			printf("Port B1 = %x\n",ReadPortB1(d));
			printf("Port C1 = %x\n",ReadPortC1(d));
			WriteCW(d,0x04);

			stat=2;
			ioctl(d, LAPCI_IOCTL_ENAIRQ, &stat); /* Enable PLX IRQ2 */
		} 
    }
	printf("Done\n");

	close(d);
	return 0;
}
